/* -*-mode:C++-*- */
/*
 * ksi_jump.h
 * ksi long jumps
 *
 * Copyright (C) 1997-2000,2001, Ivan Demakov
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose, without fee, and without a written agreement
 * is hereby granted, provided that the above copyright notice and this
 * paragraph and the following two paragraphs appear in all copies.
 * Modifications to this software may be copyrighted by their authors
 * and need not follow the licensing terms described here, provided that
 * the new terms are clearly indicated on the first page of each file where
 * they apply.
 *
 * IN NO EVENT SHALL THE AUTHORS OR DISTRIBUTORS BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
 * INCLUDING LOST PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS
 * DOCUMENTATION, EVEN IF THE AUTHORS HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * THE AUTHORS AND DISTRIBUTORS SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND THE AUTHORS AND DISTRIBUTORS HAS NO OBLIGATIONS
 * TO PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 *
 *
 * Author:        Ivan Demakov <demakov@users.sourceforge.net>
 * Creation date: Fri Aug  8 00:22:29 1997
 * Last Update:   Tue Mar 27 19:57:51 2001
 *
 *
 * $Id: ksi_jump.h,v 1.4 2001/04/04 16:59:07 demakov Exp $
 *
 */

#ifndef KSI_JUMP_H
#define KSI_JUMP_H


#ifndef KSI_H
#  include "ksi.h"
#endif

#include <setjmp.h>

#ifdef sparc
#  define FLUSH_REGISTER_WINDOWS asm("ta 3")
#else
#  define FLUSH_REGISTER_WINDOWS
#endif


typedef enum _ksi_rethrow_t
{
  KSI_RETHROW_NORMAL,
  KSI_RETHROW_ERROR,
  KSI_RETHROW_FATAL,
  KSI_RETHROW_EXIT
} ksi_rethrow_t;

struct Ksi_Jump
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  jmp_buf		j_buf;
  ksi_stack		stack;
  ksi_stack		from;
  int			size;
  ksi_obj		module;
  ksi_wind		wind;
};

struct Ksi_Catch
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  struct Ksi_Jump	jmp;
  ksi_obj		handler;
  ksi_obj		tag;
  ksi_obj		thrown;
  ksi_obj		value;
  ksi_obj		retry;
  ksi_rethrow_t		rethrow;
};

struct Ksi_Wind
{
#ifdef __cplusplus
  void* operator new (size_t sz)  { return ksi_malloc (sz); }
  void  operator delete (void* p) { ksi_free (p); }
#endif

  ksi_wind		cont;
  struct Ksi_Catch*	the_catch;
  ksi_obj		pre;
  ksi_obj		post;
};


#ifdef __cplusplus
extern "C" {
#endif

SI_API
void
ksi_init_jump (ksi_jump b, ksi_stack start, ksi_stack here);

SI_API
ksi_obj
ksi_set_jump (ksi_jump buf, ksi_stack start);

SI_API
void
ksi_long_jump (ksi_jump buf, ksi_obj val);

#ifdef __cplusplus
}
#endif

#endif
